/*****************************************************************************
 *
 *  IntListParameter.cc                         XMM Science Analysis Subsystem
 *
 *  Integer-list task parameter
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: IntListParameter.cc,v 1.15 2001/03/28 19:20:35 jbakker Exp $
 *
 *****************************************************************************/

#include <strstream>             // C++ headers
#include <iostream>
#include <string>
#include <vector>
#include <list>

#include "IntListParameter.h"             // Application headers
#include "StringToken.h"
#include <errstr.h>
#include <StringFns.h>
#include "freezestream.h"


//================================================================
//  Class IntListParameter
//================================================================

/*** Format the parameter value as a string in quotes ***/

string IntListParameter::asString() const
{
    ostrstream ostr;
    vector<int>::const_iterator it = _value.begin();
    ostr << '\'';
    if(it != _value.end())
        ostr << *it++;
    while(it != _value.end())
        ostr << ' ' << *it++;
    ostr << '\'' << ends;

    return freezestream(ostr);
}


/*** Format the parameter value as a string in quotes ***/

string IntListParameter::defaultAsString() const
{
    ostrstream ostr;
    vector<int>::const_iterator it = _default.begin();
    ostr << '\'';
    if(it != _default.end())
        ostr << *it++;
    while(it != _default.end())
        ostr << ' ' << *it++;
    ostr << '\'' << ends;

    return freezestream(ostr);
}


/***  Parse string representation and assign to 'value' if it is valid ***/

bool IntListParameter::parse(const string& str, vector<int>& value)
{
    list<string> stringTokens = StringFns::splitString(string(str)," \t");

    bool ok = true;
    list<string>::const_iterator it = stringTokens.begin();
    while(ok && it != stringTokens.end())
    {
        string s = *it++;
	StringToken t(s);
	int i = t.asInt();
	if(t.isValid())
	    value.push_back(i);
	else {
	  errstr << "Parameter '" << _name << "'" << Sas::error("ParamWrongType");
	  ok = false;
	}
    }
    return ok;
}


/*** Write a text representation of the constraint ***/

void IntListParameter::writeConstraint(ostream& os) const
{
    os << '(';
    if(_min.isDefined())
        os << _min << "<=";
    os << _name;
    if(_max.isDefined())
        os << "<=" << _max;
    os << ")";
}


/*** Revert to default value ***/

void IntListParameter::revertToDefault()
{
    _value = _default;
    notifyView();
}


/*** Test whether the parameter is valid ***/

bool IntListParameter::validate() const
{
     return isValid(_value);
}


/*** Test whether value is allowable for the parameter ***/

bool IntListParameter::isValid(const vector<int>& value) const
{
    bool valid = true;
    vector<int>::const_iterator it = value.begin();
    while(it != value.end())
    {
        int v = *it++;
	if(_min.isDefined())
	    valid &= (v >= _min);
	if(_max.isDefined())
	    valid &= (v <= _max);
    }
    return valid;
}


/*** Set the parameter to the specified value ***/

bool IntListParameter::setValue(const vector<int>& value)
{
    bool valid = isValid(value);
    if(valid)
    {
        _value = value;
	setDefined();
	notifyView();
    }
    return valid;
}


/*** Set the default to the specified value ***/

bool IntListParameter::setDefault(const vector<int>& value)
{
    bool valid = isValid(value);
    if(valid)
        _default = value;
    return valid;
}


/*** Set the value, specified as a string ***/

bool IntListParameter::setNewValue(const string& str)
{
    vector<int> values;
    bool ok =  parse(str, values) && setValue(values);

    if(!ok)
       reportRangeError(str);

    notifyView();
    return true;
}


/***  Set the default, specified as a string ***/

bool IntListParameter::setNewDefault(const string& str)
{
    vector<int> values;
    bool ok = parse(str, values) && setDefault(values);

    if(!ok)
        reportRangeError(str);

    return true;
}


/*** Return the length of the list ***/

int IntListParameter::size() const
{
    return _value.size();
}


/*** Report that value is out of range ***/

void IntListParameter::reportRangeError(const string& valueString)
{
  errstr << "Parameter '" << _name << "';" << endl
	 << "value '" << valueString << "' is outside range ";
  writeConstraint(errstr);
  errstr << Sas::error("ParamNotInRange");
}
