/*****************************************************************************
 *
 *  Parameter.cc                               XMM Science Analysis Subsystem
 *
 *  Parameter of a task
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: Parameter.cc,v 1.17 2001/03/28 19:20:35 jbakker Exp $
 *
 *****************************************************************************/

#include "Parameter.h"         // Application headers
#include "StringToken.h"

#include <iostream>          // C++ headers
#include <errstr.h>
#include <vector>
#include <string>
#include <list>


//================================================================
//  Abstract Class Parameter
//================================================================

/*** Usage and implementation of views is in praivate class ***/
class Parameter::ParameterViews : public list<ParameterView* > {};;

Parameter::Parameter(const string& name = "", const string& prompt = "")
  : _name(name), _prompt(prompt), 
    _isSet(false), _isMandatory(false), _views(0)
{
  _views = new ParameterViews;
}

Parameter::Parameter(const Parameter& p)
  : _name(p._name), _prompt(p._prompt), 
    _isSet(p._isSet), _isMandatory(p._isMandatory), _views(0)
{
  _views = new ParameterViews;
  *_views=*(p._views);
}


Parameter::~Parameter()
{
  delete _views;
}

void Parameter::setView(ParameterView* view)
{
  _views->clear();
  if (view) _views->push_back(view);
}

void Parameter::addView(ParameterView* view)
{
  if (view) _views->push_back(view);  
}

void Parameter::deleteView(ParameterView* view=0)
{
  if (view) {_views->remove(view);}
  else {_views->clear();}
}
void Parameter::notifyView()
{
  ParameterViews::const_iterator it;
  for (it=_views->begin();it!=_views->end();it++) (*it)->updateView();
}

void Parameter::wrongType() const
{
  errstr << "Parameter '" << _name << "' is accessed with wrong type" 
         << Sas::error("ParamWrongType");
}

bool Parameter::boolValue() const
{
    wrongType();
    return false;
}

int Parameter::intValue() const
{
    wrongType();
    return 0;
}

double Parameter::realValue() const
{
    wrongType();
    return 0;
}

const string& Parameter::stringValue() const
{
    wrongType();
    return _emptyString;    // Cannot return reference to temporary
}

vector<int> Parameter::boolListValue() const
{
    wrongType();
    return _emptyBoolList;
}

vector<int> Parameter::intListValue() const
{
    wrongType();
    return _emptyIntList;
}

vector<double> Parameter::realListValue() const
{
    wrongType();
    return _emptyDoubleList;
}

vector<string> Parameter::stringListValue() const
{
    wrongType();
    return _emptyStringList;
}

ostream& operator<<(ostream& os, const Parameter& p)
{
    return os << p.asString();
}


/*** Write help about parameter type, defaults, etc ***/

void Parameter::writeHelpToStream(ostream& os) const
{
  os << "  " << _name << ':' << getTypeName() << ' ';
    writeConstraint(os);
    os << ", default=" << defaultAsString();
    if(!_prompt.empty())
        os << " (" << _prompt << ')';
    os << endl;
}


/*** Initialisation of static constatnts ***/

const string          Parameter::_emptyString;
const vector<int>     Parameter::_emptyBoolList;  // Temporary work-around for egcs
const vector<int>     Parameter::_emptyIntList;
const vector<double>  Parameter::_emptyDoubleList;
const vector<string>  Parameter::_emptyStringList;

bool Parameter::setValue(const Parameter* p)
{
  if (this == p) return false;
  if (getType() != p->getType() || getType() == Undefined) return false;

  return assignValueOf(*p);
}

