/*****************************************************************************
 *
 *  RealParameter.cc                            XMM Science Analysis Subsystem
 *
 *  Real task parameter
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: RealParameter.cc,v 1.12 2001/03/28 19:20:36 jbakker Exp $
 *
 *****************************************************************************/

#include <strstream>             // C++ headers

#include "RealParameter.h"          // Application headers
#include "StringToken.h"
#include <errstr.h>
#include <iostream>
#include "freezestream.h"


//================================================================
//  Class RealParameter
//================================================================


/*** Format the parameter value as a string ***/

string RealParameter::asString() const
{
    ostrstream ostr;
    ostr.precision(12);
    ostr << _value << ends;
    return freezestream(ostr);
}


/*** Format the default value as a string ***/

string RealParameter::defaultAsString() const
{
    ostrstream ostr;
    ostr.precision(12);
    ostr << _default << ends;
    return freezestream(ostr);
}


/***  Parse string representation and assign to 'value' if it is valid ***/

bool RealParameter::parse(const string& str, double& value)
{
    StringToken t(str);
    double n = t.asDouble();
    if(t.isValid())
        value = n;
    else
      errstr << "Parameter '" << _name << "'" << Sas::error("ParamWrongType");

    return true;
}


/*** Write a text representation of the constraint ***/

void RealParameter::writeConstraint(ostream& os) const
{
    os << '(';
    if(_min.isDefined())
        os << _min << "<=";
    os << _name;
    if(_max.isDefined())
        os << "<=" << _max;
    os << ")";
}


/*** Revert to default value ***/

void RealParameter::revertToDefault()
{
    _value = _default;
    notifyView();
}


/*** Test whether the parameter is valid ***/

bool RealParameter::validate() const
{
    return isValid(_value);
}


/*** Test whether value is allowable for the parameter ***/

bool RealParameter::isValid(double n) const
{
    return (!_min.isDefined() || (n >= _min))
	&& (!_max.isDefined() || (n <= _max));
}


/*** Set the parameter to the specified value ***/

bool RealParameter::setValue(double value)
{
    bool valid = isValid(value);
    if(valid)
      {
        _value = value;
	setDefined();
	notifyView();
      }
    return valid;
}


/*** Set the default to the specified value ***/

bool RealParameter::setDefault(double value)
{
    bool valid = isValid(value);
    if(valid)
        _default = value;
    return valid;
}


/*** Set the value, specified as a string ***/

bool RealParameter::setNewValue(const string& str)
{
    double value;
    bool ok = parse(str, value) && setValue(value);

    if(!ok)
        reportRangeError(str);

    notifyView();
    return true;
}


/*** Set the default, specified as a string ***/

bool RealParameter::setNewDefault(const string& str)
{
    double value;
    bool ok = parse(str, value) && setDefault(value);

    if(!ok)
        reportRangeError(str);

    return true;
}


/*** Report that value is out of range ***/

void RealParameter::reportRangeError(const string& valueString)
{
  errstr << "Parameter '" << _name << "';" << endl
	 << "value '" << valueString << "' is outside range ";
  writeConstraint(errstr);
  errstr << Sas::error("ParamNotInRange");
}


