/*****************************************************************************
 *
 *  StringParameter.cc                          XMM Science Analysis Subsystem
 *
 *  String task parameter
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: StringParameter.cc,v 1.11 2000/06/23 09:20:45 jbakker Exp $
 *
 *****************************************************************************/

#include <string>                      // C++ headers

#include "StringParameter.h"           // Application headers
#include <errstr.h>


//================================================================
//  Class StringParameter
//================================================================

/*** Format the parameter value as a string in quotes ***/

string StringParameter::asString() const
{
    return '\'' +_value + '\'';
}


/*** Format the default value as a string ***/

string StringParameter::defaultAsString() const
{
    return '\'' +_default + '\'';
}


/*** Write a text representation of the constraint ***/

void StringParameter::writeConstraint(ostream& os) const
{
    if(!_choices.empty())
    {
        os << '{';
	list<string>::const_iterator it = _choices.begin();
	if(it != _choices.end())
	    os << *it++;
	while(it != _choices.end())
	    os << ' ' << *it++;
	os << '}';
    }
}


/***  Parse string representation and assign to 'value' if it is valid ***/

bool StringParameter::parse(const string& str, string& value)
{
    value = str;
    return true;
}


/*** Revert to default value ***/

void StringParameter::revertToDefault()
{
    _value = _default;
    notifyView();
}


/*** Test whether the parameter is valid ***/

bool StringParameter::validate() const
{
    return isValid(_value);
}


/***  Test whether value is allowable for the parameter ***/

bool StringParameter::isValid(const string& value) const
{
    // Check that the value is in the allowed set

    if(!_choices.empty())
    {
        bool ok = false;
	list<string>::const_iterator it = _choices.begin();
	while(it != _choices.end())
	{
	    if(*it++ == value)
	        ok = true;
	}
	return ok;
    } else {
        return true;
    }
}


/*** Set the parameter to the specified value ***/

bool StringParameter::setValue(const string& value)
{
    bool valid = isValid(value);
    if(valid)
    {
        _value = value;
	setDefined();
	notifyView();
    }
    return valid;
}


/*** Set the default to the specified value ***/

bool StringParameter::setDefault(const string& value)
{
    bool valid = isValid(value);
    if(valid)
        _default = value;

    return valid;
}


/*** Set the value, specified as a string ***/

bool StringParameter::setNewValue(const string& str)
{
    string value;
    bool ok = parse(str, value) && setValue(value);

    if(!ok)
        reportRangeError(str);

    notifyView();
    return true;
}


/*** Set the default, specified as a string ***/

bool StringParameter::setNewDefault(const string& str)
{    
    string value;
    bool ok = parse(str, value) && setDefault(value);

    if(!ok)
        reportRangeError(str);

    return true;
}


/*** Report that value is out of range ***/

void StringParameter::reportRangeError(const string& valueString)
{
  errstr << "Parameter '" << _name << "';" << endl
	 << "value '" << valueString << "' is outside range ";
  writeConstraint(errstr);
  errstr << Sas::error("ParamNotInRange");
}
