/*****************************************************************************
 *
 *  TimeFormatter                               XMM Science Analysis Subsystem
 *
 *  Time with methods for parsing and formatting
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: TimeFormatter.cc,v 1.5 2001/03/28 19:20:38 jbakker Exp $
 *
 *****************************************************************************/

#include "TimeFormatter.h"
#include "STime.h"
#include "Xmm.h"
#include "StringFns.h"

#include <iostream>
#include <iomanip>
#include "freezestream.h"


//================================================================
//  Class TimeFormatter
//================================================================

/*** Constructor ***/

TimeFormatter::TimeFormatter()
{
    _mode = MRT;
}


TimeFormatter::TimeFormatter(TimeMode mode)
{
    _mode = mode;
}


/*** Set the format mode ***/

void TimeFormatter::setMode(TimeMode mode)
{
    _mode = mode;
}


/*** Format the parameter value as a string ***/

string TimeFormatter::asString(double value) const
{
    string result;
    try {
        STime::errorHandling = STime::EXCEPTION;
	switch(_mode)
        {
            case MRT: {
	        ostrstream ostr;
			ostr << setprecision(10)
				 << value << ends;
			result = freezestream(ostr);
			break;
	    }
            case JD: {
	        STime t = Xmm::missionReferenceTime + value;
		ostrstream ostr;
		ostr.setf(ios::fixed,ios::floatfield);
		ostr << setprecision(10)
		     << "jd" << t.julianDay().full() << ends;
		string s = freezestream(ostr);

		// Remove trailing zeroes
		string::size_type i = s.find_last_not_of('0');
		if(i == string::npos)
		    result = "0";
		else
		    result = s.substr(0,i+1);
	      break;
	    }
            case MJD: {
	        STime t = Xmm::missionReferenceTime + value;
		ostrstream ostr;
		ostr.setf(ios::fixed,ios::floatfield);
		ostr << setprecision(10)
		     <<"mjd" << t.modifiedJulianDay().full() << ends;
		string s = freezestream(ostr);
		
		// Remove trailing zeroes
		string::size_type i = s.find_last_not_of('0');
		if(i == string::npos)
		    result = "0";
		else
		    result = s.substr(0,i+1);
		break;
	    }
            case CAL: {
	        STime t = Xmm::missionReferenceTime + value;
		result = t.toString(3,STime::CALENDAR);
		break;
	    }
            case FITS: {
	        STime t = Xmm::missionReferenceTime + value;
		result = t.toString(3,STime::FITS);
		break;
	    }
            default:
                break;
	}
    } catch(...) { }

    return result;
}


/***  Parse string representation and assign to 'value' if it is valid ***/

bool TimeFormatter::parse(const string& str, double& value)
{
    bool ok = false;
    TimeMode mode = MRT;

    try {
        STime::errorHandling = STime::EXCEPTION;

	int len = str.length();
	switch(str[0]) {

	    // Seconds since mission reference time: e.g. "123.45"
	    // or FITS format: e.g. "1997-10-13T00:00:00.000"
            case '0':
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
	    case '9':
	    case '-': {

	        // If it is a valid decimal number, treat it is seconds
	        // since mission reference time.
	        double n = 0;
		if(StringFns::parseDouble(str, n))
		{
		    value = n;
		    mode  = MRT;
		    ok = true;
		} else {           // otherwise as FITS
		    STime t(str);
		    ok = true;
		    mode = FITS;
		    value = t - Xmm::missionReferenceTime;
		}
		break;
	    }

	    // Decimal year: e.g. "y1999.1234"
//	    case 'y': {
//  	        double n = 0;
//  		if(StringFns::parseDouble(str.substr(1), n))
//  		{
//  		    STime t(n, STime::YEAR);
//  		    ok = true;
//  		    value = t - Xmm::missionReferenceTime;
//  		}
//  		break;
//  	    }

            // Julian day: e.g. "jd2450734.5"
	    case 'j': {
	        if(len >= 2 && str[1] == 'd')
		{
		    double n = 0;
		    if(StringFns::parseDouble(str.substr(2), n) && n >= JDMIN)
		    {
		        STime t(n, STime::JD);
			ok = true;
			mode = JD;
			value = t - Xmm::missionReferenceTime;
		    }
		}
		break;
	    }

            // Modified Julian day: e.g. "mjd50734"
	    case 'm': {
	        if(len >= 3 && str[1] == 'j' && str[2] == 'd')
		{
		    double n = 0;
		    if(StringFns::parseDouble(str.substr(3), n) && n >= 0)
		    {
		        STime t(n, STime::MJD);
			ok = true;
			mode = MJD;
       			value = t - Xmm::missionReferenceTime;
		    }
		}
		break;
	    }

	    // Calendar format: e.g. "Sun Oct 16 01:03:52 1997"
	    default: {
		STime t(str);
		ok = true;
		mode = CAL;
		value = t - Xmm::missionReferenceTime;
	        break;
	    }
	}
    } catch(...) { }

    if(ok) {
        _mode = mode;
    }

    return ok;
}

