/*****************************************************************************
 *
 *  TimeFormatter                               XMM Science Analysis Subsystem
 *
 *  Time with methods for parsing and formatting
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: TimeFormatter.h,v 1.3 2001/03/28 19:20:38 jbakker Exp $
 *
 *****************************************************************************/

#ifndef TIMEFORMATTER_H
#define TIMEFORMATTER_H

#include <string>              // C++ headers
#include <strstream>


//================================================================
//  Class TimeFormatter
//
//  TimeFormatter handles parsing and unparsing of time/date values
//  in a variet of formats:
//
//     xxxx-xx-xxTxx:xx[:xx.xxx]  (FITS date and time)
//     jdxxxxxx.xxx               (Julian day)
//     mjdxx.xxx                  (modified Julian day)
//     yxxxx.xxx                  (decimal year - Not yet implemented)
//     [-]xxxx[.xxx]              (seconds since XMM reference time)
//     xxx xxx xx xx:xx:xx xxxx   (calendar format)
//
//  where `x' is a single digit, `m','j',`d' and `T' are literal characters
//  and square brackets denote optional items. The number of digits shown
//  is not significant.
//
// Examples:
//
//     1997-10-13T00:00:00.000    (FITS date and time)
//     jd2450734.5                (Julian day)
//     mjd50734                   (modified Julian day)
//     -6.912e+06                 (seconds since XMM reference time)
//     Mon Oct 13 00:00:00 1997   (calendar format)
//
//================================================================

class TimeFormatter
{
public:
    enum TimeMode { MRT, JD, MJD, CAL, FITS };

    // Constructor
    TimeFormatter();
    TimeFormatter(TimeMode mode);

    // Format the time as a string
    string asString(double value) const;

    // Parse the string and set the time
    bool parse(const string& str, double& value);

    void setMode(TimeMode mode=MRT);
    TimeMode mode() { return _mode; }

private:

    // Member data
    TimeMode _mode;

    // Constants
    static const double JDMIN = 2400000.0;
};

#endif

