/*****************************************************************************
 *
 *  param_impl.cc                               XMM Science Analysis Subsystem
 *
 *  Implements F90 interface to parameter system
 *
 *  (c) 1998  ESA  -  Written by Marco Beijersbergen
 *
 *  $Id: param_impl.cc,v 1.20 2001/03/28 19:20:39 jbakker Exp $
 *
 *****************************************************************************/


#include <cassert>		// C

#include <string>		// C++

#include <Tracer.h>		// SAS
#include <stringutil.h>
#include <f90cxx.h>

#include "param.h"


F90PROCDEF(Logical,booleanparameter)(
		const char *		name,
		const int *		pos,
		unsigned		nameLen )
{
  PROC
  bool b;
  if (pos)
    b = booleanParameter(string(name,nameLen), *pos);
  else
    b = booleanParameter(string(name,nameLen));
  RETURN b;
}

F90PROCDEF(int,intparameter)(
		const char *		name,
		const int *		pos,
		unsigned		nameLen )
{
  PROC
  int i;
  if (pos)
    i = intParameter(string(name,nameLen),*pos);
  else
    i = intParameter(string(name,nameLen));
  RETURN i;
}

F90PROCDEF(double,realparameter)(
		const char *		name,
		const int *		pos,
		unsigned		nameLen )
{
  PROC
  double d;
  if (pos)
    d = realParameter(string(name,nameLen),*pos);
  else
    d = realParameter(string(name,nameLen));
  RETURN d;
}


F90PROCDEF(void,stringparameter)(
		char *			val,
		unsigned 		valLen,
		const char *		name,
		const int *		pos,
		unsigned 		nameLen )
{
  PROC

  string param;

  if (pos) 
    param = stringParameter(string(name,nameLen),*pos);
  else
    param = stringParameter(string(name,nameLen));

  unsigned paramLen = param.length();
  unsigned n = min(valLen,paramLen);
  strncpy(val,param.c_str(),n);
  memset(val+n,' ',valLen-n);
  ENDPROC
}

F90PROCDEF(int,parametercount)(
		const char *		name,
		unsigned		nameLen )
{
  PROC
  RETURN parameterCount(string(name,nameLen));
}

F90PROCDEF(void,openparameters)(const char *category, unsigned categoryLen)
{
  PROC
  openParameters(string(category,categoryLen));
  ENDPROC
}

F90PROCDEF(void,closeparameters)()
{
  PROC
  closeParameters();
  ENDPROC
}

F90PROCDEF(void,setbooleanparameter)(const char* name, Logical& value, unsigned nameLen)
{
  PROC
  setParameter(string(name,nameLen),bool(value));
  ENDPROC
}

F90PROCDEF(void,setintparameter)(const char* name, int& value, unsigned nameLen)
{
  PROC
  setParameter(string(name,nameLen),value);
  ENDPROC
}

F90PROCDEF(void,setreal32parameter)(const char* name, float& value, unsigned nameLen)
{
  PROC
  setParameter(string(name,nameLen),value);
  ENDPROC
}

F90PROCDEF(void,setreal64parameter)(const char* name, double& value, unsigned nameLen)
{
  PROC
  setParameter(string(name,nameLen),value);
  ENDPROC
}

F90PROCDEF(void,setstringparameter)(const char* name, const char* value, unsigned nameLen, unsigned valueLen)
{
  PROC
  setParameter(string(name,nameLen),string(value,valueLen));
  ENDPROC
}

F90PROCDEF(void,setbooleanlistparameter)(const char* name, const Dope<Logical,1>& dope, unsigned nameLen)
{
  PROC
  vector<Logical> value(
	dope.addr,
	dope.addr+dope.dim[0].extent);
  // Work-around for egcs compiler problem with vector<bool>
  vector<int> bvalue;
  for (unsigned i = 0; i < value.size(); ++i)
    bvalue.push_back(value[i]);
  setParameter(string(name,nameLen), bvalue );
  ENDPROC
}

F90PROCDEF(void,setintlistparameter)(const char* name, const Dope<int,1>& dope, unsigned nameLen)
{
  PROC
  vector<int> value(dope.addr, dope.addr+dope.dim[0].extent);
  SAS_DEBUG( for (unsigned i = 0; i < value.size(); ++i) clog << value[i] << endl; )

  setParameter(string(name,nameLen), value);
  ENDPROC
}

F90PROCDEF(void,setreal64listparameter)(const char* name, const Dope<double,1>& dope, unsigned nameLen)
{
  PROC
  vector<double> value(dope.addr, dope.addr+dope.dim[0].extent);

  setParameter(string(name,nameLen), value);
  ENDPROC
}

F90PROCDEF(void,setreal32listparameter)(const char* name, const Dope<float,1>& dope, unsigned nameLen)
{
  PROC
  vector<float> value(dope.addr, dope.addr+dope.dim[0].extent);
  vector<double> fvalue;
  for (unsigned i = 0; i < value.size(); ++i) fvalue.push_back(value[i]);
  setParameter(string(name,nameLen), fvalue);
  ENDPROC
}


typedef ChDope<1> StringVector;
F90PROCDEF(void,setstringlistparameter)(const char* name, const ChDope<1>& dope, unsigned nameLen)
{
  PROC
  vector<string> value;
  for (int i = 0; i < dope.dim[0].extent; ++i)
    value.push_back(trim(string(dope.addr+dope.dim[0].mult*i,dope.len)));
  setParameter(string(name,nameLen),value);
  ENDPROC
}

